import _cpeg, struct

code = [(_cpeg.Any, 0, 1),
        (_cpeg.Capture, 0, 2, 5),
        (_cpeg.Any, 0, 1),
        (_cpeg.End,)]
instr = ''.join([_cpeg.pack(*c) for c in code])
print _cpeg.match(instr, u"..")

code = [(_cpeg.Call, 2),
 (_cpeg.Jump, 11),
 (_cpeg.Char, 7, u'O'),
 (_cpeg.Choice, 6, 1),
 (_cpeg.Char, 0, u'm'),
 (_cpeg.Char, 0, u'e'),
 (_cpeg.Char, 0, u'g'),
 (_cpeg.Char, 0, u'a'),
 (_cpeg.Commit, 3),
 (_cpeg.Any, 0, 1),
 (_cpeg.Jump, -8),
 (_cpeg.Return,),
 (_cpeg.End,)]

instr = ''.join([_cpeg.pack(*c) for c in code])

print _cpeg.match(instr, u"...Omega...")
import time
txt = open("kjv10.txt").read().decode('ascii')
print "CPEG"
for i in range(10):
    start = time.clock()
    print _cpeg.match(instr, txt),
    print "%.2f sec" % (time.clock() - start)

print "String.find"
for i in range(10):
    start = time.clock()
    print txt.find("Omega"),
    print "%.2f sec" % (time.clock() - start)

print "re.find"
import re
r = re.compile(r"Omega")
for i in range(10):
    start = time.clock()
    m = r.search(txt)
    if m:
        print m.end(),
    else:
        print "Not found",
    print "%.2f sec" % (time.clock() - start)




# -*- coding: latin1 -*-
"""A port of the lpeg test suite to ppeg.

This is a direct port of as much of the lpeg test suite as makes sense for
the ppeg port of lpeg.
"""

# Shorthand for the pattern type
from _ppeg import Pattern as P
import sys
from nose.tools import *

# General tests for the ppeg library
def test_type():
    assert type("alo") != P
    assert type(sys.stdin) != P
    assert type(P.Match("alo")) == P

# Tests for some basic optimizations
def test_basic_opt_choice():
    p = P.Fail() | P.Match("a")
    assert p("a").pos == 1
    p = P() | P.Match("a")
    assert p("a").pos == 0
    p = P.Match("a") | P.Fail()
    assert not p("b")
    p = P.Match("a") | P()
    assert p("b").pos == 0

def test_basic_opt_concat():
    p = P.Fail() + P.Match("a")
    assert not p("a")
    p = P() + P.Match("a")
    assert p("a").pos == 1
    p = P.Match("a") + P.Fail()
    assert not p("a")
    p = P.Match("a") + P()
    assert p("a").pos == 1

def test_basic_opt_assert():
    p = +P.Fail() + P.Match("a")
    assert not p("a")
    p = +P() + P.Match("a")
    assert p("a").pos == 1
    p = P.Match("a") + +P.Fail()
    assert not p("a")
    p = P.Match("a") + +P()
    assert p("a").pos == 1

# Locale tests not included yet

def test_match_any():
    assert P.Any(3)("aaaa")
    assert P.Any(4)("aaaa")
    assert not P.Any(5)("aaaa")
    assert P.Any(-3)("aa")
    assert not P.Any(-3)("aaa")
    assert not P.Any(-3)("aaaa")
    assert not P.Any(-4)("aaaa")
    assert P.Any(-5)("aaaa")

def test_match_match():
    assert P.Match("a")("alo").pos == 1
    assert P.Match("al")("alo").pos == 2
    assert not P.Match("alu")("alo")
    assert P()("").pos == 0

# Character sets
def cs2str(c):
    # d = c.dump()
    # assert len(d) == 2, "cs2str: pattern is not a charset"
    # if d[0][0] == 'fail':
    #     return ""
    # if d[0][0] == 'char':
    #     return chr(d[0][1])
    # assert d[0][0] == 'set', "cs2str: pattern is not a charset (" + d[0][0] + ")"
    # return d[0][3]
    return "".join(chr(ch) for ch in range(256) if c(chr(ch)))
def eqcharset(c1, c2):
    assert cs2str(c1) == cs2str(c2)

digit = P.Set("0123456789")
upper = P.Set("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
lower = P.Set("abcdefghijklmnopqrstuvwxyz")
letter = P.Set("") | upper | lower
alpha = letter | digit | P.Range("")

def test_charset():
    eqcharset(P.Set(""), P.Fail())
    eqcharset(upper, P.Range("AZ"))
    eqcharset(lower, P.Range("az"))
    eqcharset(upper | lower, P.Range("azAZ"))
    eqcharset(upper | lower, P.Range("AZczaabb90"))
    eqcharset(digit, P.Set("01234567") | P.Match("8") | P.Match("9"))
    eqcharset(upper, letter - lower)
    eqcharset(P.Set(""), P.Range(""))
    assert cs2str(P.Set("")) == ""
    eqcharset(P.Set("\0"), P.Match("\0"))
    eqcharset(P.Set("\1\0\2"), P.Range("\0\2"))
    eqcharset(P.Set("\1\0\2"), P.Range("\1\2") | P.Match("\0"))
    eqcharset(P.Set("\1\0\2") - P.Match("\0"), P.Range("\1\2"))

word = alpha ** 1 + (P.Any(1) - alpha) ** 0

def test_word():
    assert (word ** 0 + P.Any(-1))("alo alo")
    assert (word ** 1 + P.Any(-1))("alo alo")
    assert (word ** 2 + P.Any(-1))("alo alo")
    assert not (word ** 3 + P.Any(-1))("alo alo")
    assert not (word ** -1 + P.Any(-1))("alo alo")
    assert (word ** -2 + P.Any(-1))("alo alo")
    assert (word ** -3 + P.Any(-1))("alo alo")

eos = P.Any(-1)

def test_eos():
    assert (digit ** 0 + letter + digit + eos)("1298a1")
    assert not (digit ** 0 + letter + eos)("1257a1")

b = P.Grammar(P.Match("(") +
              (((P.Any(1) - P.Set("()")) | +P.Match("(") + P.Var(0)) ** 0) +
              P.Match(")"))

def test_matching_paren():
    assert b("(a1())()")
    assert not (b + eos)("(a1())()")
    assert (b + eos)("((a1())()(é))")
    assert not b("(a1()()")

def test_word_exclusion():
    assert not (letter ** 1 - P.Match("for"))("foreach")
    assert (letter ** 1 - (P.Match("for") + eos))("foreach")
    assert not (letter ** 1 - (P.Match("for") + eos))("for")

# Now we need capture support...
def basiclookfor(p):
    return P.Grammar(p | 1+P.Var(0))

def caplookfor(p):
    return basiclookfor(P.Cap(p))

def test_initial_matches():
    b = P.Grammar(P.Match("(") +
              (((P.Any(1) - P.Set("()")) | +P.Match("(") + P.Var(0)) ** 0) +
              P.Match(")"))
    assert caplookfor(letter**1)("   4achou123...").captures == ["achou"]
    a = (caplookfor(letter**1)**0)(" two words, one more  ").captures
    assert a == ["two", "words", "one", "more"]
    assert basiclookfor((+b + 1) + P.CapP())("  (  (a)").pos == 6
    m = (P.Cap(digit**1 + P.CapC("d")) | P.Cap(letter**1 + P.CapC("l")))("123")
    assert_equals(m.captures, ["123", "d"])
    m = (P.Cap(digit**1) + "d" + (-1) | P.Cap(letter**1 + P.CapC("l")))("123d")
    assert_equals(m.captures, ["123"])
    m = (P.Cap(digit**1 + P.CapC("d")) | P.Cap(letter**1 + P.CapC("l")))("abcd")
    assert_equals(m.captures, ["abcd", "l"])



from _ppeg import Pattern as P

class Var:
    def __getattr__(self, name):
        return P.Var(name)
V = Var()

EndOfFile = -P(1)
EndOfLine = P("\r\n") | P("\r") | P("\n")
Space = P(" ") | P("\t") | V.EndOfLine
Comment = P("#") + (-V.EndOfLine + P(1)) ** 0 + V.EndOfLine
Spacing = (V.Space | V.Comment) ** 0

LEFTARROW = P("<-") + V.Spacing
SLASH = P("/") + V.Spacing
AND = P("&") + V.Spacing
NOT = P("!") + V.Spacing
QUESTION = P("?") + V.Spacing
STAR = P("*") + V.Spacing
PLUS = P("+") + V.Spacing
OPEN = P("(") + V.Spacing
CLOSE = P(")") + V.Spacing
DOT = P(".") + V.Spacing

Char = ( P("\\") + P.Set("nrt'\"*[]\\") |
       P("\\") + P.Range("02") + P.Range("07") + P.Range("07") |
       P("\\") + P.Range("07") + P.Range("07") ** -1 |
       -P("\\") + P(1) )

Range = V.Char + P("-") + V.Char | V.Char + V.Spacing
Class = P("[") + ( -P("]") + V.Range ) ** 0 + P("]") + V.Spacing
Literal = (P("'") + ( -P("'") + V.Char ) ** 0 + P("'") + V.Spacing |
          P('"') + ( -P('"') + V.Char ) ** 0 + P('"') + V.Spacing
          )

Identifier = V.IdentStart + V.IdentCont ** 0 + V.Spacing
IdentStart = P.Range("azAZ") | P("_")
IdentCont = V.IdentStart | P.Range("09")

Primary = (V.Identifier + -V.LEFTARROW |
          V.OPEN + V.Expression + V.CLOSE |
          V.Literal |
          V.Class |
          V.DOT) + V.Spacing

Suffix = (V.Primary + 
          (V.QUESTION | V.STAR | V.PLUS) ** -1 +
          V.Spacing)
Prefix = (V.AND | V.NOT) ** -1 + V.Suffix
Sequence = V.Prefix ** 0
Expression = V.Sequence + (V.SLASH + V.Sequence) ** 0

Definition = V.Identifier + V.LEFTARROW + V.Expression

Grammar = V.Spacing + V.Definition ** 1 + V.EndOfFile

PEG = P.Grammar(
        start = "Grammar",
        Grammar = Grammar,
        Definition = Definition,
        Expression = Expression,
        Sequence = Sequence,
        Prefix = Prefix,
        Suffix = Suffix,
        Primary = Primary,
        Identifier = Identifier,
        IdentStart = IdentStart,
        IdentCont = IdentCont,
        Literal = Literal,
        Class = Class,
        Range = Range,
        Char = Char,
        LEFTARROW = LEFTARROW,
        SLASH = SLASH,
        AND = AND,
        NOT = NOT,
        QUESTION = QUESTION,
        STAR = STAR,
        PLUS = PLUS,
        OPEN = OPEN,
        CLOSE = CLOSE,
        DOT = DOT,
        Spacing = Spacing,
        Comment = Comment,
        Space = Space,
        EndOfLine = EndOfLine,
        EndOfFile = EndOfFile
)

# Spacing rules need work!
PEG_GRAMMAR = r"""
# Hierarchical syntax
Grammar <- Spacing Definition+ EndOfFile
Definition <- Identifier LEFTARROW Expression
Expression <- Sequence (SLASH Sequence)*
Sequence <- Prefix*
Prefix <- (AND / NOT)? Suffix
Suffix <- Primary (QUESTION / STAR / PLUS)?
Primary <- Identifier !LEFTARROW
/ OPEN Expression CLOSE
/ Literal / Class / DOT
# Lexical syntax
Identifier <- IdentStart IdentCont* Spacing
IdentStart <- [a-zA-Z_]
IdentCont <- IdentStart / [0-9]
Literal <- ['] (!['] Char)* ['] Spacing
/ ["] (!["] Char)* ["] Spacing
Class <- '[' (!']' Range)* ']' Spacing
Range <- Char '-' Char / Char
Char <- '\\' [nrt'"\[\]\\]
/ '\\' [0-2][0-7][0-7]
/ '\\' [0-7][0-7]?
/ !'\\' .
LEFTARROW <- '<-' Spacing
SLASH <- '/' Spacing
AND <- '&' Spacing
NOT <- '!' Spacing
QUESTION <- '?' Spacing
STAR <- '*' Spacing
PLUS <- '+' Spacing
OPEN <- '(' Spacing
CLOSE <- ')' Spacing
DOT <- '.' Spacing
Spacing <- (Space / Comment)*
Comment <- '#' (!EndOfLine .)* EndOfLine
Space <- ' ' / '\t' / EndOfLine
EndOfLine <- '\r\n' / '\n' / '\r'
EndOfFile <- !.
"""



"""A test implementation of a PEG program builder in Python.

The PEG matcher needs to be in C for speed, but the instruction stream can be
built in Python and comprezssed into a C-style structure at the last minute,
just before matching. This allows for higher-level access to the instructions,
and hence more flexibility in experimenting with and implementing operations
and optimisations.
"""

# Opcodes
Opcodes = [
  "IAny", "IChar", "ISet", "ISpan",
  "IRet", "IEnd",
  "IChoice", "IJmp", "ICall", "IOpenCall",
  "ICommit", "IPartialCommit", "IBackCommit", "IFailTwice", "IFail",
  "IGiveup", "IFunc",
  "IFullCapture", "IEmptyCapture", "IEmptyCaptureIdx",
  "IOpenCapture", "ICloseCapture", "ICloseRunTime"
]

class Op(object):
    pass
Op = Op()

for n, op in enumerate(Opcodes):
    setattr(Op, op, n)

class Instruction(object):
    def __init__(self, op, aux = None, offset = 0):
        self.op = op
        self.aux = aux
        self.offset = offset
    def __str__(self):
        return "%s(%s,%s)" % (Opcodes[self.op], self.aux, self.offset)

def Fail():
    return [Instruction(Op.IFail)];

def Succeed():
    return []

def Any(n):
    # Breaking up into 255-chunks is implementation dependent. Maybe leave
    # this until converting to final form???
    if n == 0:
        return []
    if n > 0:
        ret = []
        while (n > 255):
            ret.append(Instruction(Op.IAny, 255))
            n -= 255
        ret.append(Instruction(Op.IAny, n))
        return ret
    else:
        ret = []
    return [Instruction(Op.IAny, n)]

def Str(s):
    return [Instruction(Op.IChar, c) for c in s]



import timeit
setup="""
from _ppeg import Pattern

text = open('kjv10.txt').read()

pat = Pattern.Dummy()
pat.dump()
"""

t = timeit.Timer(stmt='print pat(text).pos', setup=setup)
N=5
print "Time taken: %.2f sec" % (t.timeit(N)/float(N))




from distutils.core import setup, Extension

setup (
    name='PPeg',
    version='0.9',
    description="A Python port of Lua's LPeg pattern matching library",
    url='https://bitbucket.org/pmoore/ppeg',

    author='Paul Moore',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2 :: Only',
        'Topic :: Text Processing :: General',
    ],

    keywords='parsing peg grammar regex',

    ext_modules = [Extension('_ppeg', ['_ppeg.c', 'lpeg.c']),
                   Extension('_cpeg', ['_cpeg.c'])],
    py_modules=[
        'PythonImpl',
        'pegmatcher',
    ],
)




from __future__ import with_statement
from unittest import TestCase, main
from _ppeg import Pattern as P
import sys
from cStringIO import StringIO
from contextlib import contextmanager

@contextmanager
def stdout(fd):
    old = sys.stdout
    sys.stdout = fd
    yield fd
    sys.stdout = old

class TestBasic(TestCase):
    def testpat(self):
        p = P()
        self.assertEqual(type(p), P)
    def testdisplay(self):
        p = P.Any(1)
        with stdout(StringIO()) as s:
            p.display()
        self.assertNotEqual(s.getvalue(), '')

class TestEquivalents(TestCase):
    def testfail(self):
        self.assertEqual(P(None), P.Fail())
    def testany(self):
        self.assertEqual(P(1), P.Any(1))
        self.assertEqual(P(1000), P.Any(1000))
        self.assertEqual(P(-1), P.Any(-1))
        self.assertEqual(P(-1000), P.Any(-1000))
        self.assertEqual(P(0), P.Any(0))
    def testmatch(self):
        self.assertEqual(P("a"), P.Match("a"))
        self.assertEqual(P("abcdef"), P.Match("abcdef"))
        self.assertEqual(P(""), P.Match(""))
        self.assertEqual(P("a\0b"), P.Match("a\0b"))
    def testset(self):
        self.assertEqual(P(set=""), P.Set(""))
        self.assertEqual(P(set="ab12"), P.Set("ab12"))
        self.assertEqual(P(set="ab\0\1\2"), P.Set("ab\0\1\2"))
        self.assertEqual(P(set="abc"), P(set=u"abc"))
    def testrange(self):
        self.assertEqual(P(range=""), P.Range(""))
        self.assertEqual(P(range="azAZ"), P.Range("azAZ"))
        self.assertEqual(P(range="\0\255"), P.Range("\0\255"))
        self.assertEqual(P(range="ac"), P(range=u"ac"))

class TestInitChecks(TestCase):
    def testtoomany(self):
        self.assertRaises(TypeError, P, 1, set="ab")
        self.assertRaises(TypeError, P, 1, range="ab")
        self.assertRaises(TypeError, P, range="az", set="ab")
    def testtype(self):
        self.assertRaises(TypeError, P, set())
        # Maybe this should work (by converting to string, like set/range)
        self.assertRaises(TypeError, P, u"unicode")
        self.assertRaises(TypeError, P, 1.5)
    def testset(self):
        self.assertRaises(TypeError, P, set=12)
        self.assertRaises(TypeError, P, set=None)
    def testrange(self):
        self.assertRaises(TypeError, P, range=12)
        self.assertRaises(TypeError, P, range=None)
        self.assertRaises(ValueError, P, range=u"abc")
        self.assertRaises(ValueError, P, range="abc")

class TestBuild(TestCase):
    def match(self, pat, items):
        self.assertEqual([i[0] for i in pat.dump()], items + ['end'])
    def testany(self):
        self.match(P.Any(0), [])
        self.match(P.Any(1), ['any'])
        # Negative self.match - any & fail
        self.match(P.Any(-1), ['any', 'fail'])
        # Overflow - max of 255 per opcode
        self.match(P.Any(300), ['any', 'any'])
    def testmatch1(self):
        self.match(P.Match('a'), ['char'])
    def testmatch2(self):
        self.match(P.Match('ab'), ['char', 'char'])
    def testfail(self):
        self.match(P.Fail(), ['fail'])
    def testset(self):
        p = P.Set("abdef")
        self.match(p, ['set'])
        self.assertEqual([p(s).pos for s in "abcdefgh"],
                [1, 1, -1, 1, 1, 1, -1, -1])
    def testrange(self):
        p = P.Range("bceg")
        self.match(p, ['set'])
        self.assertEqual([p(s).pos for s in "abcdefgh"],
                [-1, 1, 1, -1, 1, 1, 1, -1])

class TestConcat(TestCase):
    def testany(self):
        p1 = P.Any(2)
        p2 = P.Any(1) + P.Any(1)
        self.assertEqual(p1.dump(), p2.dump())
    def testid(self):
        p1 = P.Dummy()
        p2 = P() + p1
        self.assertEqual(p1.dump(), p2.dump())
        p2 = p1 + P()
        self.assertEqual(p1.dump(), p2.dump())
    def testfail(self):
        p1 = P.Dummy()
        p2 = P.Fail() + p1
        self.assertEqual(p2.dump(), P.Fail().dump())
        p2 = p1 + P.Fail()
        self.assertEqual(p2.dump(), P.Fail().dump())
    def testmatch(self):
        splits = [
            ('a', 'bcd'),
            ('ab', 'cd'),
            ('abc', 'd'),
            ('a', '', 'bcd'),
            ('a', 'b', 'cd'),
            ('a', 'bc', 'd'),
            ('ab', '', 'cd'),
            ('ab', 'c', 'd'),
            ('abc', '', 'd'),
            ('a', 'b', 'c', 'd'),
        ]
        p1 = P.Match("abcd")
        for split in splits:
            p2 = P.Match(split[0])
            for s in split[1:]:
                p2 = p2 + P.Match(s)
            self.assertEqual(p1.dump(), p2.dump())
            p2 = P.Match('')
            for s in split:
                p2 = p2 + P.Match(s)
            self.assertEqual(p1.dump(), p2.dump())
            p2 = P.Match(split[0])
            for s in split[1:]:
                p2 = p2 + P.Match(s)
            p2 = p2 + P.Match('')
            self.assertEqual(p1.dump(), p2.dump())

class TestAnd(TestCase):
    def match(self, pat, items):
        self.assertEqual([i[0] for i in pat.dump()], items + ['end'])
    def testtrue(self):
        self.match(+P(), [])
    def testfail(self):
        self.match(+P.Fail(), ['fail'])
    def testanycset(self):
        # &Any(1) is a 0-length match of any character. This is
        # optimised as a match against a character set with every bit set -
        # if the match succeeds, the pattern fails.
        self.match(+P.Any(1), ['set', 'fail'])
    def testother(self):
        self.match(+P.Any(5), ['choice', 'any', 'back_commit', 'fail'])
    def testlonger(self):
        p = P.Any(1) + P.Match('ab')
        self.match(+p, ['choice', 'any', 'char', 'char', 'back_commit', 'fail'])

class TestPow(TestCase):
    # TODO: Add more tests!!!
    def match(self, pat, items):
        self.assertEqual([i[0] for i in pat.dump()], items + ['end'])
    def testsimple(self):
        self.match(P.Any(1) ** 3, ['any', 'any', 'any', 'span'])
    def testmatch(self):
        p = P.Match("foo") ** 0
        self.assertEqual(p("boofoo").pos, 0)
        self.assertEqual(p("foofoo").pos, 6)

class TestDiff(TestCase):
    def match(self, pat, items):
        self.assertEqual([i[0] for i in pat.dump()], items + ['end'])
    def testtrue(self):
        self.match(-P(), ['fail'])
    def testfail(self):
        self.match(-P.Fail(), [])
    def testdiff(self):
        # Not an obvious translation - the optimizer hits us. This has been
        # validated against the Lua lpeg implementation
        p = P.Match('bc') - P.Match('ef')
        self.match(p, ['char', 'choice', 'char', 'failtwice', 'char', 'char'])

class TestCapture(TestCase):
    def captype(self, n, p):
        return p.dump()[n][1] & 0xF
    def match(self, pat, items):
        self.assertEqual([i[0] for i in pat.dump()], items + ['end'])
    def testfullcap(self):
        p = P.Cap(P.Any(1))
        self.match(p, ['any', 'fullcapture'])
        off = p.dump()[1][2]
        # Csimple == 5
        self.assertEqual(self.captype(1, p), 5)
        self.assertEqual(off, 0)
    def testopenclose(self):
        p = P.Cap(P.Any(1)**1)
        self.match(p, ['any', 'opencapture', 'span', 'closecapture'])
        off = p.dump()[1][2]
        # Csimple == 5
        self.assertEqual(self.captype(1, p), 5)
        self.assertEqual(off, 0)
    def testtypes(self):
        self.assertEqual(self.captype(0, P.CapP()), 1)
        self.assertEqual(self.captype(0, P.CapA(1)), 4)
        self.assertEqual(self.captype(1, P.CapT(P.Any(1))), 6)
        self.assertEqual(self.captype(1, P.CapS(P.Any(1))), 10)

class TestMatch(TestCase):
    def testdummy(self):
        p = P.Dummy()
        self.assertEqual(p("aOmega").pos, 6)
    def testany(self):
        p = P.Any(0)
        self.assertEqual(p("fred").pos, 0)
        p = P.Any(1)
        self.assertEqual(p("fred").pos, 1)
        p = P.Any(-1)
        self.assertEqual(p("fred").pos, -1)
        p = P.Any(-1)
        self.assertEqual(p("").pos, 0)
    def testmatch(self):
        p = P.Match("fred")
        self.assertEqual(p("fred").pos, 4)
        self.assertEqual(p("freddy").pos, 4)
        self.assertEqual(p("jim").pos, -1)
    def testfail(self):
        p = P.Fail()
        self.assertEqual(p("jim").pos, -1)
        self.assertEqual(p("").pos, -1)

class TestCaptureRet(TestCase):
    def testpos(self):
        p = P.Any(3) + P.CapP() + P.Any(2) + P.CapP()
        self.assertEqual(p("abcdef").captures, [3, 5])
    def testarg(self):
        p = P.CapA(1) + P.CapA(3) + P.CapA(2)
        self.assertEqual(p("abcdef", 1, "hi", None).captures, [1, None, "hi"])
    def testconst(self):
        p = P.CapC(["foo", 1, None, "bar"])
        self.assertEqual(p("abcdef").captures, [["foo", 1, None, "bar"]])
    def testsimple(self):
        p = P.Any(1) + P.Cap(P.Any(2))
        self.assertEqual(p("abc").captures, ["bc"])
        self.assertEqual(P.Cap(p)("abc").captures, ["abc", "bc"])
    def testsubst(self):
        p = P.Any(1) + P.CapS(P.CapP() + P.Any(2))
        self.assertEqual(p("abc").captures, ["1bc"])
        self.assertEqual(P.Cap(p)("abc").captures, ["abc", "1bc"])

class TestOpenCall(TestCase):
    def testopencall(self):
        p = P.Var("Unset")
        self.assertRaises(RuntimeError, p, "test")

class TestGrammar(TestCase):
    def testsimple(self):
        p = P.Any(1)
        pg = P.Grammar(p)
        self.assertEqual(pg("ab").pos, 1)
    def testnamedrule(self):
        p = P.Grammar(P.Var("a") + P.Var("a"), a=P(1))
        self.assertEqual(p("ab").pos, 2)
    def testexplicitstart(self):
        p = P.Grammar(P.Var("a") + P.Var("a"), a=P(1), start="a")
        self.assertEqual(p("ab").pos, 1)
    def testpositionalrules(self):
        p = P.Grammar(P.Var(1) + P.Var(2), P(1), P(1))
        self.assertEqual(p("ab").pos, 2)
    def testleftrecursion(self):
        self.assertRaises(RuntimeError, P.Grammar, P.Var(0) + P(1))

class TestDummy(TestCase):
    def testbuilddummy(self):
        patt = P.Grammar(P.Match("Omega") | P.Any(1) + P.Var(0))
        d = P.Dummy()
        self.assertEqual(patt.dump(), d.dump())

def lines(str):
    return [l.strip() for l in str.strip().splitlines()]

class TestCaptureBug(TestCase):
    def testbug001(self):
        p = P.Grammar(P.Match("Omega") | P.Any(1) + P.Var(0))
        p2 = P.CapC("hello") + p | P.CapC(12)
        with stdout(StringIO()) as s:
            p2.display()
        expected = """\
00: choice -> 15 (0)
01: emptycaptureidx constant(n = 0)  (off = 1)
02: call -> 4
03: jmp -> 14
04: char 'O'-> 11
05: choice -> 11 (1)
06: char 'm'-> FAIL
07: char 'e'-> FAIL
08: char 'g'-> FAIL
09: char 'a'-> FAIL
10: commit -> 13
11: any * 1-> FAIL
12: jmp -> 4
13: ret
14: commit -> 16
15: emptycaptureidx constant(n = 0)  (off = 3)
16: end
"""
        result = s.getvalue()
        for l1, l2 in zip(lines(result), lines(expected)):
            self.assertEqual(l1, l2)

class TestSubclass(TestCase):
    class T(P):
        pass
    def testcreation(self):
        T = self.T
        self.assertEqual(T(1), P(1))
        self.assertEqual(type(T(1)), T)
        self.assert_(isinstance(T(1), P))
    def testassert(self):
        T = self.T
        self.assert_(isinstance(+T(1), T))
    def testnegate(self):
        T = self.T
        self.assert_(isinstance(-T(1), T))
    def testconcat(self):
        T = self.T
        self.assert_(isinstance(T(1) + T(1), T))
        self.assert_(isinstance(T(1) + P(1), T))
    def testchoice(self):
        T = self.T
        self.assert_(isinstance(T(1) | T(1), T))
        self.assert_(isinstance(T(1) | P(1), T))
    def testdiff(self):
        T = self.T
        self.assert_(isinstance(T(1) - T(1), T))
        self.assert_(isinstance(T(1) - P(1), T))
    def testpow(self):
        T = self.T
        self.assert_(isinstance(T(1) ** 1, T))

class TestCoercion(TestCase):
    def testconcat(self):
        self.assertEqual(P(1)+1, 1+P(1))
        self.assertEqual(P("a")+1, "a"+P(1))
        self.assertEqual(P(None)+1, None+P(1))
    def testchoice(self):
        self.assertEqual(P(1)|1, 1|P(1))
        self.assertEqual(P("a")|1, "a"|P(1))
        self.assertEqual(P(None)|1, None|P(1))
    def testdiff(self):
        self.assertEqual(P(1)-1, 1-P(1))
        self.assertEqual(P("a")-1, "a"-P(1))
        self.assertEqual(P(None)-1, None-P(1))

class TestGroupCap(TestCase):
    def testsimple(self):
        self.assertEqual(P.CapG(P(1))("a").captures, ["a"])
    def testnamed(self):
        self.assertEqual(P.CapG(P(1), "dummy")("a").captures, [])
    def testbackref(self):
        p = P.CapG(P(1), "dummy") + P.CapB("dummy") + P.CapB("dummy")
        self.assertEqual(p("a").captures, ["a", "a"])
    def testunknownbackref(self):
        p = P.CapG(P(1), "dummy") + P.CapB("unknown")
        self.assertRaises(RuntimeError, p, "a")

class TestOtherCap(TestCase):
    def testquery(self):
        tbl = dict(a="fail", bc="result", ab="fail")
        p = P(1) + P.Cap(P(2))
        self.assertEqual((p/tbl)("abcdef").captures, ["result"])
    def testfunction(self):
        def fn(s):
            return s.upper()
        p = P(1) + P.Cap(P(2))
        self.assertEqual((p/fn)("abcdef").captures, ["BC"])
    def testfold(self):
        def fn(a, v):
            return a + ", " + v.upper()
        p = P.CapF(P(1) + (P.Cap(P(2))**1), fn)
        # Note: Initial value is unchanged!
        self.assertEqual(p("abcdefg").captures, ["bc, DE, FG"])
    def testfolddummyaccum(self):
        def fn(a, v):
            a.append(v.upper())
            return a
        p = P.CapF(P.CapC([]) + P(1) + (P.Cap(P(2))**1), fn)
        self.assertEqual(p("abcdefg").captures, [["BC", "DE", "FG"]])

class TestRuntimeCap(TestCase):
    def testbasic(self):
        matchone = P.Cap(P(1))
        # Icky interface :-(
        def fn(str, rest, pos, caps):
            if (rest.startswith(caps[0])):
                return pos+len(caps[0])
            return None
        matchtwo = P.CapRT(matchone, fn)
        self.assertEqual(matchtwo("aa").pos, 2)
        self.assertEqual(matchtwo("aab").pos, 2)
        self.assertEqual(matchtwo("ab").pos, -1)

if __name__ == '__main__':
    main()
